<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\CountryAndLanguage;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the ISO 3166-1 alpha-3 code.
 *
 * @property-read string $value The raw code.
 */
final class ISO3166Alpha3 implements \Stringable {
	/**
	 * The code value of the country.
	 *
	 * @var string
	 */
	private readonly string $value;

	/**
	 * The constructor.
	 *
	 * @param mixed $value The continent of the country.
	 * @throws \InvalidArgumentException If the id is not numeric or less than 1.
	 */
	public function __construct( mixed $value ) {
		if ( ! is_string( $value ) ) {
			throw new \InvalidArgumentException( 'The value for ISO3166-1:alpha3 must be a string.' );
		}
		$value = trim( $value );
		if ( strlen( $value ) !== 3 ) {
			throw new \InvalidArgumentException( 'The value for ISO3166-1:alpha3 must be exactly 3 characters long.' );
		}
		$value = strtoupper( $value );
		if ( ! ctype_alpha( $value ) ) {
			throw new \InvalidArgumentException( 'The value for ISO3166-1:alpha3 must be alphabetic.' );
		}

		$this->value = $value;
	}

	/**
	 * The magick getter.
	 *
	 * @param string $name The name of the property.
	 * @throws \InvalidArgumentException If the property does not exist.
	 */
	public function __get( string $name ): mixed {
		return match ( $name ) {
			'value' => $this->value,
			default => throw new \InvalidArgumentException( 'The property "' . esc_html( $name ) . '" does not exist.' ),
		};
	}

	/**
	 * The magick to string method.
	 */
	public function __toString(): string {
		return $this->value;
	}
}
